<?php

/****************************************************************************
name:		count-clever.php
version:	1.0
copyright:	(c) 2004 Carsten Meier

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

contact:	Carsten Meier
		Under de Diek 19
		D-26757 Borkum
		Germany

homepage:	http://www.trexity.de/
e-mail:		trexity@t-online.de/


counterdb-file-format:

header:
	0x00 signature ('COUNTCLEVER1')
	0x0c counter-value (unsigned-64-bit-BE integer)
	0x14 sorted array of ip-entries (most recent first)

ip-entry:
	0x00 time (unsigned-32-bit-BE integer)
	0x04 ip4addr (unsigned-32-bit-BE integer)
	0x08 end of entry
****************************************************************************/


define('COUNTERDB_SIGNATURE', 'COUNTCLEVER1');
define('COUNTERDB_HEADER_SIZE', 20);
define('COUNTERDB_IP_ENTRY_SIZE', 8);


// initializes counter-database-file with counter-value

function init_counter($counterdbname, $counterval)
{
	if(!($handle = fopen($counterdbname, 'wb')))
		return false;
	flock($handle, LOCK_EX);

	// write new database-file

	$buf = pack('a12NN', COUNTERDB_SIGNATURE, 0, $counterval);
	fwrite($handle, $buf);

	// clean up

	flock($handle, LOCK_UN);
	fclose($handle);

	return true;
}


// simply returns counter-value w/o increasing it

function get_counter($counterdbname)
{
	if(!($handle = fopen($counterdbname, 'rb')))
		return NULL;
	flock($handle, LOCK_SH);

	// read header

	$buf = fread($handle, COUNTERDB_HEADER_SIZE);
	if(strlen($buf)==COUNTERDB_HEADER_SIZE)
	{
		$val = unpack('a12sig/Ncounterhi/Ncounter', $buf);
		if($val['sig'] != COUNTERDB_SIGNATURE)
		{
			flock($handle, LOCK_UN);
			fclose($handle);
			return NULL;
		}
		return $val['counter'];
	}
	else
	{
		flock($handle, LOCK_UN);
		fclose($handle);
		return NULL;
	}
}


// arguments: counter-database-filename, entry-livetime in seconds, and
// if this call is a visit that should be counted.
// returns array with the elements 'visits' and 'online' or NULL on error.

function process_counter($counterdbname, $duration, $isvisit)
{
	$curtime = time();
	$remoteip = ip2long($_SERVER['REMOTE_ADDR']);

	if(!($handle = fopen($counterdbname, 'r+b')))
		return NULL;
	flock($handle, LOCK_EX);

	// read header

	$visits = 0;

	$buf = fread($handle, COUNTERDB_HEADER_SIZE);
	if(strlen($buf)==COUNTERDB_HEADER_SIZE)
	{
		$val = unpack('a12sig/Ncounterhi/Ncounter', $buf);
		if($val['sig'] != COUNTERDB_SIGNATURE)
		{
			flock($handle, LOCK_UN);
			fclose($handle);
			return NULL;
		}

		$visits = $val['counter'];
		unset($val);
	}
	else
	{
		flock($handle, LOCK_UN);
		fclose($handle);
		return NULL;
	}

	// scan ip-table

	$online = 0;
	$ipfound = false;
	$iptabbuf = '';

	while($buf = fread($handle, COUNTERDB_IP_ENTRY_SIZE))
	{
		if(strlen($buf) != COUNTERDB_IP_ENTRY_SIZE)
			break;

		$val = unpack('Ntime/Nip', $buf);

		// skip entries, if this entry is stale
		if($curtime - $val['time'] >= $duration)
			break;

		if($isvisit && $remoteip==$val['ip'])
		{
			$ipfound = true;
			continue;
		}

		// insert entry into updated database
		$iptabbuf .= $buf;

		++$online;
	}

	if($isvisit)
	{
		if(!$ipfound)
			++$visits;
		++$online;
	}

	// write new database-file

	rewind($handle);

	// write header

	fwrite($handle, pack('a12NN', COUNTERDB_SIGNATURE, 0, $visits));

	// write ip-table

	if($isvisit)
		fwrite($handle, pack('NN', $curtime, $remoteip));
	fwrite($handle, $iptabbuf);

	ftruncate($handle, ftell($handle));

	// clean up

	flock($handle, LOCK_UN);
	fclose($handle);

	return array('visits'=>$visits, 'online'=>$online);
}


// transforms string into a graphical representation
// arguments: format-string with %s where the digit should appear,
//	string to transform

function make_graphical($str, $fmt)
{
	$ret = '';
	$len = strlen($str);
	for($x = 0; $x < $len; ++$x)
		$ret .= sprintf($fmt, $str{$x});
	return $ret;
}


?>